//
//        Source: hidrelay.c
//         Title: CLI tool to set/reset USB/HID relay device
//
// Last Modified: 2023-01-02 09:21:31
//
// Copyright (C) 2006-2009, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY Keith Edwin Smith ''AS IS'' AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Keith Edwin Smith BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "hidrelay.h"
#include "string.h"
#include "local.h"

char *argvlist[] = {
    "--off"
    ,"--on"
    ,"--rename"
    ,"--show"
    ,"--help"
    ,"-q"
    ,"--quiet"
    ,"--query"
    ,NULL
};

#define ARGV_OFF 0
#define ARGV_ON 1
#define ARGV_RENAME 2
#define ARGV_SHOW 3
#define ARGV_HELP 4
#define ARGV_Q 5
#define ARGV_QUIET 6
#define ARGV_QUERY 7
#define ARGV_NULL 8

struct global_struct {
    char *relay_device_id;
    char *relay_device_newid;
    int state[MAX_RELAYS];
    int action;
    int noise;
};
typedef struct global_struct GLOBALS;
GLOBALS g;

#define ACTION_SHOW 0
#define ACTION_SET 1
#define ACTION_RENAME 2
#define ACTION_QUERY 3

//----------------------------------------------------------------------
// USAGE
//----------------------------------------------------------------------
int usage(char *pgm) {
    fprintf(stderr,
            "\nUsage: %s [flags] [device ID]\n"
            "  CLI to manipulate relays on a USB HID relay control\n"
            "  [device ID] is the name of the HID device to use\n"
            "    If only one device present it will use that one\n"
            "  Default action will show current devices and relay states\n"
            "\n"
            "  flags:\n"
            "    -q|--quiet       No Noise\n"
            "    --on {n}         Turn on relay {n}\n"
            "    --off {n}        Turn on relay {n}\n"
            "    --help           This text\n"
            "    --show           For symmetry (default action)\n"
            "    --query {n}      Show status for relay {n}\n"
            "    --rename {old} {new}\n"
            "                   Rename device from {old} to {new}\n"
            "                   (5 char limit I think)\n"
            ,pgm
            );
    exit(0);
}

//----------------------------------------------------------------------
// PARSE_ARG_NEXT
//----------------------------------------------------------------------
char *parse_arg_next(int argc, char *argv[], int arg_ix, char *last_arg) {
    if(arg_ix >= argc) {
        fprintf(stderr,"Error, could not get argument for %s\n",last_arg);
        usage(argv[0]);
    }
    return(argv[arg_ix]);
}
//----------------------------------------------------------------------
// PARSE_ARGS
//----------------------------------------------------------------------
int parse_args(int argc, char *argv[]) {
    int relay_number;
    int ix, arg_ix;

    g.noise = 1;
    for(ix = 1; ix < argc ; ix++) {
        arg_ix = argvindex(argvlist,argv[ix],strcmp);
        switch(arg_ix) {
        case ARGV_OFF:
        case ARGV_ON:
        case ARGV_QUERY:
            relay_number = atol(parse_arg_next(argc,argv,++ix,argv[arg_ix])) - 1;
            if(relay_number > MAX_RELAYS) {
                fprintf(stderr,"%d is not a valid relay number\n",relay_number);
                exit(-1);
            }
            switch(arg_ix) {
            case ARGV_OFF:
                g.action = ACTION_SET;
                g.state[relay_number] = HIDRELAY_OFF;
                break;
            case ARGV_ON:
                g.action = ACTION_SET;
                g.state[relay_number] = HIDRELAY_ON;
                break;
            case ARGV_QUERY:
                g.action = ACTION_QUERY;
                g.state[relay_number] = HIDRELAY_QUERY;
                g.noise = 0;
                break;
            }
            break;
        case ARGV_HELP:
            usage(argv[0]);
            break; // NOTREACHED
        case ARGV_Q:
        case ARGV_QUIET:
            g.noise = 0;
            break;
        case ARGV_RENAME:
            g.relay_device_id = strdup(argv[++ix]);
            g.relay_device_newid = strdup(argv[++ix]);
            g.action = ACTION_RENAME;
            break;
        case ARGV_SHOW:
            g.action = ACTION_SHOW;
            break;
        default:
            g.relay_device_id = strdup(argv[ix]);
            break;
        }
    }
    return(0);
}


//----------------------------------------------------------------------
// MAIN
//----------------------------------------------------------------------
int main(int argc, char *argv[]) {
    HIDRELAY *hid_first, *hr;
    int relay_device_count = 0;
    int ix;

    memset(&g,0,sizeof(g));
    parse_args(argc,argv);

    hid_first = hidrelay_find(1); // keep open
    for(hr = hid_first, relay_device_count = 0; hr != NULL; hr = hr->next)
        relay_device_count++;
    if(g.relay_device_id != NULL) {
        for(hr = hid_first; hr != NULL; hr = hr->next) {
            if(strcmp(hr->name,g.relay_device_id) == 0) {
                break;
            }
        }
        if(hr == NULL) {
            fprintf(stderr,"Could not locate relay device named %s\n"
                    ,g.relay_device_id);
            exit(-1);
        }
    } else if(relay_device_count > 1) {
        fprintf(stderr,"Could not locate relay device named %s\n"
                ,g.relay_device_id);
        exit(-1);
    } else {
        hr = hid_first;
    }
    switch(g.action) {
    case ACTION_SHOW:
        hidrelay_show(hr, HIDRELAY_SHOW_STATE);
        break;
    case ACTION_QUERY:
        for(ix = 0; ix < hr->relay_count; ix++) {
            if(g.state[ix] == HIDRELAY_QUERY) {
                fprintf(stdout,"%s\n",hidrelay_status(hr->state[ix]));
            }
        }
        break;
    case ACTION_SET:
        for(ix = 0; ix < hr->relay_count; ix++) {
            hr->request[ix] = g.state[ix];
        }
        if(g.noise) {
            hidrelay_show(hr, HIDRELAY_SHOW_REQUEST);
        }
        hidrelay_set(hr);
        break;
    }
    exit(0);
}
