/*
	scan.c - Scan for a list of strings
		adapted from standalone scan program
		by Keith E. Smith	3/12/91

	usage: scan [-v] [-f filename] [-timeout] string string ....

	scan reads stdin until it discovers a match with one of the
		supplied strings or until timeout seconds go by. If
		-v is supplied on the command line scan will echo stdin
		to stderr. -f supplies a filename to store input in.
		
	NOTE:  Timouts will not work properly on buffered input

	Revision History:
	09/18/91	Added the -v flag to allow echoing of input
			from stdin to stderr.

	01/25/92	Added some comments,  Added the -f argument
			to allow for trapping input to a file.  Moved
			stderr and file output to an scan_in_char() routine
			which will also handle an alarm exit.

	01/26/92	Added recognition of standard escapes in strings,
			\\, \r, \n, \t, \b, \xxx, etc. thru a function
			fix_string which returns the length of the
			'fixed' string

*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <signal.h>

char	*buffer;
int	waitlen;
int	scan_count;
char	**scan_buffers;
int	alarm_flag;
/*
	This is to ensure that no errors occur on 0 length only strings
*/
int	longest_string = 0;
int	shortest_string = 255;
FILE	*trap_file = NULL;
char	*trap_file_name;
int	verbose = 0;


int fix_string(s)
char	*s;
{
	char	*p;
	int	val;
	int	length;

	length = 0;
	for(p = s; *s; s++) {
		length++;
		if (*s != '\\') {
			*p++ = *s;
			continue;
		}
/*
	OKAY,  We have an escape ...
*/
		switch(*++s) {
		case 'n' :
			*p++ = '\n';
			break;
		case 'r' :
			*p++ = '\r';
			break;
		case 't' :
			*p++ = '\t';
			break;
		case 'b':
			*p++ = '\b';
			break;
		case '\\':
			*p++ = '\\';
			break;
		case 0: /* Uh Oh ! */
			fprintf(stderr,"Bad String ends in \\ !");
			exit(0);
		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			val = 0;
			while(*s >= '0' && *s <= '9') {
				val = (val * 8) + (*s++ - '0');
			}
			s--;
			*p++ = val;
			break;
		default: /* Unknown Literal Then */
			*p++ = *s;
			break;
		} /* switch */
	} /* for */
	*p = 0;
	return(length);
}

alrmtrap() {
	signal(SIGALRM,SIG_DFL);
	alarm_flag = 1;
}

/*
	scan_substr - Simple locate of q in p returns 0 if not found
		no error checking, single scan.  Ought to substitute
		my more complex library function.
*/
int scan_substr(p,q)
char	*p,*q;
{
	while( *p && *p != *q) p++;
	if(!*p) return(0);
	while(*p && *q && (*p == *q)) {
	 	p++;
	 	q++;
	}
	if(*q) return(0);
	return(1);
}

/*
	scan_in_char - Take a character from stdin and if neccessary, bounce
		it back to stderr, and/or trap_file.  Exit if an alarm
		has expired while waiting for input.
*/
int scan_in_char(f,verbose,trap_file)
FILE	*f;
{
	int cc;

	cc = fgetc(f) ;

/*
	Uh Oh!  We timed out
*/
	if(alarm_flag || cc == -1) return(-1);
/*
	If flags or a trap file specified send the input to them.
*/
	if(verbose) putc(cc, stderr);
	if(trap_file != NULL) putc(cc,trap_file);

	return(cc);
}

scan(f, trap_file, verbose, timeout, strings)
FILE	*f;
FILE	*trap_file;
int	verbose, timeout;
char *strings[];
{
	char	*p, *waitstr;
	char	**sbuf;
	int	i;
	int	j;
	timeout = 30;
	scan_count = 0;
/*
	Allocate pointers to scan strings,  We'll allocate as many pointers
	as arguments plus a couple, this may result in a couple of extra
	pointers being allocated.  Big Deal still cheaper and more flexible
	than pre-dimensioning an array of MAX_STRING_COUNT or something.
*/
	for(i = 0;strings[i] != NULL; i++);
	sbuf = scan_buffers = (char **)malloc(sizeof(char *) * (i + 1));
	waitstr = "NOT DEFINED";

/*
	Here's where we load the scan arguments into an argv style array
*/
	for(i = 0;strings[i] != NULL; i++) {
		*sbuf = strdup(strings[i]);
		j = fix_string(*sbuf++);
		if(j < shortest_string) shortest_string = j;
		if(j > longest_string) longest_string = j;
	}
	if(sbuf == scan_buffers) {
		return(0);
	}
	*sbuf = NULL;

/*
	Okay, Now do some waiting <g>

	Allocate a buffer that is the length of the longest string
	plus 2 (+2 cause I ALWAYS overflow something somewhere) we
	are scanning for.
*/
	buffer = (char *)malloc(longest_string + 2);
/*
	Begin scan, filling the scan buffer up to the length of the
	longest target string. at each input check the list of target
	strings against what's in the buffer.
*/
	signal(SIGALRM,alrmtrap);
	alarm(timeout);

	for(i=0; i < shortest_string; i++) {
		buffer[i] = scan_in_char(f,verbose,trap_file) ;
		if(buffer[i] == -1) return(0);
	}
	while(i < longest_string) {
		for (sbuf = scan_buffers,j=1; *sbuf != NULL; sbuf++,j++) {
			if(scan_substr(buffer,*sbuf)) {
				fflush(stderr);
				do_exit(j);
			}
		}
		buffer[i] = scan_in_char(f,verbose,trap_file);
		if(buffer[i++] == -1) return(0);
	}
/*
	Now that the buffer is full continue with the scan, pulling in
	characters and shifting down the data until we find a match.
*/
	while (1) {
		for (sbuf = scan_buffers,j=1; *sbuf != NULL; sbuf++,j++) {
			if(scan_substr(buffer,*sbuf)) {
				do_exit(j);
			}
		}
		for(i=0; i < (longest_string-1); i++) {
			buffer[i] = buffer[i+1] ;
		}
		buffer[i] = scan_in_char(f,verbose,trap_file);
		if(buffer[i] == -1) return(0);
	}
}
