//
//        Source: panelmonitor.h
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: Management/Polling/Logging for Energy Sensor
//
// Last Modified: 2023-01-10 09:11:47
//

#include <local.h>
#include <modbus/modbus.h>
#include <postgresql/libpq-fe.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include "relay.h"

#define DEFAULT_LOG_LEVEL 0
#define CONFIG_FILE "/etc/panelmonitor/panelmonitor.cfg"
#define RTU_DEVICE "/dev/ttyUSB0"
#define DEFAULT_INTERVAL 5
// Back to back polls will fail without some wait time
#define WAIT_NEXT_USEC 400000

#define REG_VOLTAGE 0x0000
#define REG_CURRENT_LOW 0x0001
#define REG_CURRENT_HIGH 0x0002
#define REG_POWER_LOW 0x0003
#define REG_POWER_HIGH 0x0004
#define REG_ENERGY_LOW 0x0005
#define REG_ENERGY_HIGH 0x0006
#define REG_FREQUENCY 0x0007
#define REG_POWER_FACTOR 0x0008
#define REG_ALARM 0x0009

#define ENERGY_RESET_SUCCESS 0
#define ENERGY_RESET_PENDING 1

// ==== Global variable structures ====
struct globs {
    LOG_STRUCT *log;
    int log_level;

    char *config_path;
    CONFIG_FILE_SECTION *cfg;
    time_t cfg_read_time;

    char *device_path;
    PGconn *dbh;
    modbus_t *psensor;
    int verbose;
    int action;
    char *cmd_args;
    HIDRELAY *hr_first;

    RELAY *relay[MAX_RELAYS];
    int energy_reset_month;
    int energy_reset_day;
    int energy_reset_success;
    char *energy_reset;
    int sensor_count;
    int sensor_list[256];
    int current_id;
    int new_id;
    int interval;
    int relay_min_off_time;
    int last_amps;
};

// Holds the values calculated from reading the sensor
struct sensor_values {
    int sensor_id;
    char stamp[32]; // YYYY-MM-DD HH:MM:SS.uuuuuu-0000
    double volts;
    double amps;
    double watts;
    unsigned long energy;
    double frequency;
    double power_factor;
    int alarm;
};


// Dependent declarations
#ifdef PANELMONITOR_MAIN
char *energy_reset_list[] = {
    "now"
    ,"daily"
    ,"weekly"
    ,"monthly"
    ,"yearly"
    ,NULL
};
#define EXTERN
#else
extern char *energy_reset_list[];
#define EXTERN extern
#endif
EXTERN uint16_t buf[64];
EXTERN struct sensor_values sv;
EXTERN struct globs g_args,g;

// Script actions
#define ACTION_DEFAULT 1
#define ACTION_DAEMON 0
#define ACTION_POLL_ONCE 1
#define ACTION_RESET 2
#define ACTION_SET_ID 3
#define ACTION_CALIBRATE 4
#define ACTION_SHOW 5

// Reset actions
#define RESET_NOW 0
#define RESET_DAILY 1
#define RESET_WEEKLY 2
#define RESET_MONTHLY 3
#define RESET_YEARLY 4
#define RESET_NULL 5

#define RREQ_TEST 0
#define RREQ_NOW 1
#define RREQ_STATUS 2
#define RREQ_UPDATE 3
#define RREQ_INIT 4

// Function Prototypes
#ifndef NO_PROTOTYPE
int add_table_entry(PGconn *myhouse, struct sensor_values *sv_p);
struct tm *check_periodic();
struct tm *current_timestamptz(char *buf, int buflen);
PGconn *db_open();
int end_program();
int log_relay_status(LOG_STRUCT *log,int level);
int parse_args(int argc, char *argv[]);
int parse_config();
int parse_config_relay();
int sensor_calibrate(modbus_t *psensor, int sensor_id);
int sensor_create();
int sensor_open(modbus_t *psensor, int sensor_id);
int sensor_poll_all(modbus_t *psensor, PGconn *dbh);
int sensor_poll_id(modbus_t *psensor, int sensor_id, char *stamp);
int relay_check(int amps);
int reset_check(struct tm *now, unsigned request, modbus_t *psensor, int sensor_id);
int sensor_set_id(modbus_t *psensor, int current_id, int new_id);
int sensor_reset_energy(modbus_t *psensor, int sensor_id);
int show_config(LOG_STRUCT *log, int level);
int usage(char *pgm);
#endif
