//
//
//        Source: parse_args.c
//         Title: Fetch command line arguments
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-08 10:18:26
//       Created: Wed Jan 4 20:40:10 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Modification History:
//

#include "panelmonitor.h"

// Program Arguments
char *argvlist[] = {
    "--calibrate"
    ,"--config"
    ,"-d"
    ,"--daemon"
    ,"--dev"
    ,"--device"
    ,"--help"
    ,"--id"
    ,"--interval"
    ,"-l"
    ,"--log-level"
    ,"--once"
    ,"--reset"
    ,"--reset-day"
    ,"--set-id"
    ,"--show"
    ,"-v"
    ,"--verbose"
    ,NULL
};

#define ARGV_CALIBRATE 0
#define ARGV_CONFIG 1
#define ARGV_D 2
#define ARGV_DAEMON 3
#define ARGV_DEV 4
#define ARGV_DEVICE 5
#define ARGV_HELP 6
#define ARGV_ID 7
#define ARGV_INTERVAL 8
#define ARGV_L 9
#define ARGV_LOGLEVEL 10
#define ARGV_ONCE 11
#define ARGV_RESET 12
#define ARGV_RESET_DAY 13
#define ARGV_SET_ID 14
#define ARGV_SHOW 15
#define ARGV_V 16
#define ARGV_VERBOSE 17
#define ARGV_END 18

//----------------------------------------------------------------------
// PARSE_ARGS_NEXTVAL
// Grab next string value or call usage if there is a problem
//----------------------------------------------------------------------
char *parse_args_nextval(int argc, char *argv[], int ix) {
    char *nextval;
    if(ix >= argc) {
        usage(argv[0]);
        exit(255);
    }
    nextval = strdup(argv[ix]);
    return(nextval);
}

//----------------------------------------------------------------------
// PARSE_ARGS
//----------------------------------------------------------------------
int parse_args(int argc, char *argv[]) {
    int arg,ix;

    // Set these to make the sensor reset energy periodically
    // on (maybe) the first day of the month
    g_args.action = ACTION_DEFAULT;
    g_args.verbose = 0;
    g_args.energy_reset = NULL;
    g_args.energy_reset_month = -1;
    g_args.energy_reset_day = 1;
    g_args.interval = DEFAULT_INTERVAL;

    // Parse the arguments
    for(ix = 1; ix < argc; ix++) {
        arg = argvindex(argvlist,argv[ix],strncmp);
        switch(arg) {
        case ARGV_CALIBRATE:
            g_args.action = ACTION_CALIBRATE;
            break;
        case ARGV_CONFIG:
            g_args.config_path = parse_args_nextval(argc,argv,++ix);
            break;
        case ARGV_D:
        case ARGV_DAEMON:
            g_args.action = ACTION_DAEMON;
            break;
        case ARGV_DEV:
        case ARGV_DEVICE:
            g_args.device_path = parse_args_nextval(argc,argv,++ix);
            break;
        case ARGV_HELP:
            usage(argv[0]);
            break; // NOTREACHED
        case ARGV_ID:
            // Need to parse the string instead, i.e.  1,3,5 or 11-14 . . .
            // multiple --id's for now will work
            g_args.sensor_list[g_args.sensor_count++] = strtol(argv[++ix],NULL,0);
            break;
        case ARGV_INTERVAL:
            g_args.interval = ++ix < argc ? strtol(argv[ix],NULL,0) : usage(argv[0]);
            break;
        case ARGV_L:
        case ARGV_LOGLEVEL:
            g_args.log_level = ++ix < argc ? strtol(argv[ix],NULL,0) : usage(argv[0]);
            log_open(NULL,g_args.log_level,g_args.log);
            break;
        case ARGV_ONCE:
            g_args.action = ACTION_POLL_ONCE;
            break;
        case ARGV_V:
        case ARGV_VERBOSE:
            g_args.verbose++;
            break;
        case ARGV_RESET:
            g_args.energy_reset = parse_args_nextval(argc,argv,++ix);
            break;
        case ARGV_RESET_DAY:
            g_args.energy_reset_day = ++ix < argc ? strtol(argv[ix],NULL,0) : usage(argv[0]);
            break;
        case ARGV_SET_ID:
            g_args.current_id = ++ix < argc ? strtol(argv[ix],NULL,0) : usage(argv[0]);
            g_args.new_id = ++ix < argc ? strtol(argv[ix],NULL,0) : usage(argv[0]);
            g_args.action = ACTION_SET_ID;
            break;
        case ARGV_SHOW:
            g_args.action = ACTION_SHOW;
            break;
        default:
            fprintf(stderr,"Unknown Argument: %s\n",argv[ix]);
        }
    }
    return(1);
}
