//
//
//        Source: relay_check.c
//         Title: Toggle relays based on current power reading
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-10 12:17:50
//       Created: Sun Jan 8 10:10:05 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Modification History:
//

#include "panelmonitor.h"

int log_relay_check(char *prefix,int relay,char *label,int amps,int threshold) {
    char mark;
    if(strcmp(prefix,"OFF") == 0){
        mark = '>';
    } else {
        mark = '<';
    }
    log_print(g.log,1,"TOGGLE %s: %d/%s (%d%c%d)\n"
              ,prefix, relay + 1, label, amps, mark,threshold);
}

int relay_check(int amps) {
    RELAY *rp;
    HIDRELAY *hr;
    int ix, update_needed;
    time_t curtime;

    // Clear the request array on the first relay bank
    hr = g.hr_first;
    memset(&hr->request,0,hr->relay_count);

    curtime = time(NULL);
    update_needed = 0;
    // Walk the relays for conditions
    // Turn OFF (power from grid) immediately
    // Turn ON (power from panel) if delay time met
    for(ix = 0; g.relay[ix] != NULL; ix++) {
        rp = g.relay[ix];
        if(rp->override != RELAY_OVERRIDE_NONE) {
            if(hr->state[ix] != rp->override) {
                hr->request[ix] = rp->override;
                update_needed = 1;
            }
            continue;
        }
        if(amps >= rp->threshold_off) {
            if(hr->state[ix] != HIDRELAY_OFF) {
                hr->request[ix] = HIDRELAY_OFF;
                rp->last_change = curtime;
                update_needed = 1;
                rp->current_state = RELAY_STATE_OFF;
                log_relay_check("OFF",ix, rp->label,amps, rp->threshold_off);
            }
        } else if (amps < rp->threshold_on) {
            if(hr->state[ix] != HIDRELAY_ON) {
                if((curtime - rp->last_change) > g.relay_min_off_time) {
                    hr->request[ix] = HIDRELAY_ON;
                    rp->last_change = curtime;
                    update_needed = 1;
                    rp->current_state = RELAY_STATE_ON;
                    log_relay_check("ON",ix, rp->label,amps, rp->threshold_off);
                } else {
                    if(rp->current_state != RELAY_STATE_TIME_WAIT) {
                        rp->current_state = RELAY_STATE_TIME_WAIT;
                        log_relay_check("WAIT",ix, rp->label,amps, rp->threshold_off);
                    }
                }
            }
        }
    }
    // Check for a state change, make the call if so
    if(update_needed) {
        if(!hidrelay_set(hr)) {
            log_print(g.log,0,"Failed to set relays\n");
        }
        hidrelay_fetch(hr,NULL,0,0);
    }
}
